import torch.nn as nn

class SISUGenerator(nn.Module):
    def __init__(self):
        super(SISUGenerator, self).__init__()
        layer = 16
        # Convolution layers with BatchNorm and Residuals
        self.conv1 = nn.Sequential(
            nn.Conv1d(1, layer * 2, kernel_size=7, padding=3),
            nn.BatchNorm1d(layer * 2),
            nn.PReLU(),
            nn.Conv1d(layer * 2, layer * 5, kernel_size=7, padding=3),
            nn.BatchNorm1d(layer * 5),
            nn.PReLU(),
            nn.Conv1d(layer * 5, layer * 5, kernel_size=7, padding=3),
            nn.BatchNorm1d(layer * 5),
            nn.PReLU(),
        )
        self.final_layer = nn.Sequential(
            nn.Conv1d(layer * 5, layer * 2, kernel_size=5, padding=2),
            nn.BatchNorm1d(layer * 2),
            nn.PReLU(),
            nn.Conv1d(layer * 2, 1, kernel_size=3, padding=1),
            # nn.Tanh()  # Normalize audio... if needed...
        )

    def forward(self, x):
        residual = x
        x = self.conv1(x)
        x = self.final_layer(x)
        return x + residual
